****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: createptsmasterlist.sas  
*
* Created (mm/dd/yyyy): 02/02/2017
* Last modified: 10/25/2018
* Version: 1.11
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*	This program will create the numerator dataset based on the first 4 POV datasets
*
*  Program inputs:                                                                                   
*	-The first 4 POV datasets (POV1, POV2, POV3, POV4)
* 
*  Program outputs:                                                                                                                                       
* 	-Dataset with valid numerator observations (_PtsMasterList)
* 
* PARAMETERS:
*
*  Programming Notes:                                                                                
*                                                                           
*
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials	   Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*   1.1		  05/04/2017	AP		1. Added race_out and hispanic_out cohortfile parameters
*									2. Changed Priordays_Af enrollment to (priordays_af-1) - QRP-376
*									3. Changed EnrDaysAft for Type 4 to EpisodeEndDate - QRP-377
*
*	1.2		  06/26/2017	AP		Added Type 5 analysis
*
*	1.3	      10/03/2017	AP		Added option to include episodes with 0 days post blackout (QRP-319)
*
*	1.4	      11/20/2017	AP		Removed option to include episodes with 0 days post blackout (QRP-466)
*
*	1.5	      11/15/2017	AP		Bug fix to apply EnrDaysAft in Type 3 (QRP-457)
*
*	1.6	      12/07/2017	AP		Bug fix to apply EnrDaysAft with DeathDt in Types 2 and 5 (QRP-470)
*
*	1.7	      12/21/2017	AP		1. Removed comorbidity enrollment requirement (QRP-435)
*									2. Added censoring for death in Type 1 (QRP-462)
*
*	1.8	      02/01/2018	AP		Removed post-index enrollment requirements (QRP-496)
*
*	1.9	      02/22/2018	RR		Applied same date formats to EpisodeStartDt and EpisodeEndDt (QRP-496)
*
*	1.10	  05/15/2018	AP		Added Type 6 analysis
*									Added secondary episode analysis observation window enrollment
*
*	1.11	  10/25/2018	AP		Added Type 4 infant enrollment
*
***************************************************************************************************;

%macro ms_createptsmasterlist();	

%put =====> MACRO CALLED: ms_createptsmasterlist v1.10;

	****************************************;
	* Type 1, 2, 5, or 6 cohort definition 	;
	****************************************;
	%IF %EVAL(&TYPE=1 | &TYPE=2 | &type=5 | &type=6) %THEN %DO;	   

        *Index Date Reference Cohort Criteria;
        data _PtsMasterList;
		format IndexDt EpisodeEndDt ENR_START ENR_END date9.;
        merge _POV1(in=pov1 rename=Adate=IndexDt)
              _POV2(in=pov2 rename=Adate=IndexDt)
              _POV3(in=pov3 rename=Adate=IndexDt keep=patid Adate)
              _POV4(in=pov4 rename=EpisodeStartDt=IndexDt)
              _IOT(in=iot rename=EpisodeStartDt=IndexDt);
        by Patid IndexDt; * Adate represents potential Index Date;		
        
        if pov1 and not pov2 and pov4;

		%if %eval(&type. ne 6) %then %do;
			if pov3;
		%end;

        *Index Date must overlap greater monitoring period;
        if &startfollowup. <= IndexDt <= &enddate.;

		OrigEpisEndDt = EpisodeEndDt;

		*Adjust EpisodeEndDt if EpisodeEndDt > enddate;
		%if &censor_qryend = 1 %then %do;
			if EpisodeEndDt > &enddate. then EpisodeEndDt = &enddate.;
		%end;

		*Adjust EpisodeEndDt if EpisodeEndDt > DPMAX Date;
		%if &censor_dpend = 1 %then %do;
			if EpisodeEndDt > &DP_MaxDate. then EpisodeEndDt = &DP_MaxDate.;
		%end;

		/*Adjust EpisodeEndDt if EpisodeEndDt > DeathDt*/
		if EnrEpisodeCensoredAtDeath = 1 and EpisodeEndDt > DeathDt then EpisodeEnddt = deathdt;

        *Trucate episode at IOT;
		*Only adjust for IOT when IOT prior to EpisodeEndDate. IOT after EpisodeEndDt can 
		 occur if Epiosde shortened to occur on EndDate;
        if iot and TrunkDate and trunkdate <= EpisodeEndDt then EpisodeEndDt=TrunkDate;

    	*ExpExtPer may have made the episode to go beyond elig.;
    	if EpisodeEndDt > enr_end then EpisodeEndDt = enr_end;

        if blackoutper =. then blackoutper=0;

        *Minimum look back enrollment duration;
        if Enr_Start <= IndexDt-max(0,&enrdays.,Washper,FupWashPer,&MinCovFrom.,&L1HDPSFROM.,&MinMFUfrom.) then MinEnrolMet=1;

		*take death into account for post-index enrollment;
		if EnrEpisodeCensoredAtDeath = 1 then enr_death_dt = min(enr_end, deathdt); else enr_death_dt = enr_end;		
		if enr_death_dt >= IndexDt+max(0,&EnrDaysAftInd.) then MinEnrolMet2=1;

		*Minimum enrollment after episode end;
		if Enr_End >= EpisodeEndDt + max(0, &enrdaysaftepi.) then MinEnrolMet3=1;

		*Secondary episode observation window enrollment;
		if Enr_Start <= IndexDt-max(0,-1*&obs_from_index.) then MinObsEnrolMet1=1;
		if Enr_Start <= EpisodeEndDt-max(0,-1*&obs_from_epiend.) then MinObsEnrolMet2=1;
		if Enr_End >= IndexDt+max(0,&obs_to_index.) then MinObsEnrolMet3=1;
		if Enr_End >= EpisodeEndDt + max(0, &obs_to_epiend.) then MinObsEnrolMet4=1;

        *Minimum Episode Duration;
        if EpisodeEndDt-IndexDt + 1 >= minepisdur then MinEpiDurMet=1;  

		*Maximum Episode Duration;
        if not missing(maxepisdur) and EpisodeEndDt-IndexDt + 1 > maxepisdur then EpisodeEndDt = IndexDt + maxepisdur -1;  

        *Patient must be enrolled throughout the entire minimum episode duration;
        if minepisdur > 0 then do;
            if Enr_Start <= indexDt  and Indexdt + minepisdur - 1 <= Enr_end then FUEnrolMet=1;
        end;
        else FUEnrolMet=1;

		*remove episodes that do not have at least one day at risk post blackout (at least one day to observe an event);
       	if blackoutper and EpisodeEndDt - (IndexDt + blackoutper) < 0 then delete;

		*If using PSA tool,	patient must have at least one post blackout day at risk (at least 
		one day to observe and event) in the look of their index date;
		if "&Analysis."="ps" then do;
			if blackoutper and IndexDtLookEndDt - (IndexDt + blackoutper) < 0 then delete;
		end;

        if MinEnrolMet and MinEnrolMet2 and MinEnrolMet3 and FUEnrolMet and MinEpiDurMet and MinObsEnrolMet1 and MinObsEnrolMet2 and MinObsEnrolMet3 and MinObsEnrolMet4;

    	*Defensive coding;
    	if IndexDt > EpisodeEndDt then delete;                

        keep PatID IndexDt EpisodeEndDt ENR_START ENR_END mindaysupp minepisdur maxepisdur FupWashPer blackoutper T&type.CohortDef 
				IndexLook washper RxSup RxAmt NumDispensing OrigEpisEndDt trunkdate EnrEpisodeCensoredAtDeath Deathdt enr_end_equal_death
			%if %eval(&type.=6) %then %do;
				code productapprovaldate productmarketingdate otherproductdate uptakedate
			%end;
			;
		run;
		
		%if %eval(&type=2 | &type.=6) %then %do;
			*Summarize utilization that are overlapping selected episodes;
	        %ms_shaveoutside(reffile=_PtsMasterList,
	                          refstart=IndexDt,
	                          refend=EpisodeEndDt,
	                          KeepPartBf=Y,
	                          ToShaveFile=_GroupIndex,
	                          ToShaveStart=Adate,
	                          ToShaveEnd=ExpireDt,
	                          outfile=_GroupUtilFile);

	        *Summarize RxSup during episode (only to apply mindaysupp criterion);
	        *The final summarization will be done later when episode selection is finalized;
	        proc means data=_GroupUtilFile nway noprint;
	        var RxSup RxAmt;
	        class PatId IndexDt;
	        output out=_GroupUtilFile(keep=PatId IndexDt TotRxSup) sum(RxSup)=TotRxSup;
	        run;

	        data _PtsMasterList;
	        merge _PtsMasterList(in=a)
	              _GroupUtilFile;
	        by PatId IndexDt;
	        if a;

	        if TotRxSup >= mindaysupp then MinDaySupMet=1;
	        if MinDaySupMet;

	    	drop TotRxSup washper;
	        run;
		%end;
    %END;

	*Create prevalent Type 6 episodes;
	%if %eval(&type=6) %then %do;	   

 		data _PtsMasterList2;
			format IndexDt EpisodeEndDt ENR_START ENR_END date9.;
	        merge _POV12(in=pov1 rename=Adate=IndexDt)
	              _POV3(in=pov3 rename=Adate=IndexDt keep=patid Adate)
	              _POV42(in=pov4 rename=EpisodeStartDt=IndexDt)
	              _IOT2(in=iot rename=EpisodeStartDt=IndexDt);
	        by Patid IndexDt; * Adate represents potential Index Date;		
	        
	        if pov1 and pov4;

			if pov3 then INCEXL =1; else INCEXL = 0;

	        *Index Date must overlap greater monitoring period;
	        if &startfollowup. <= IndexDt <= &enddate.;

			OrigEpisEndDt = EpisodeEndDt;

			*Adjust EpisodeEndDt if EpisodeEndDt > enddate;
			%if &censor_qryend = 1 %then %do;
				if EpisodeEndDt > &enddate. then EpisodeEndDt = &enddate.;
			%end;

			/*Adjust EpisodeEndDt if EpisodeEndDt > DeathDt*/
			if EnrEpisodeCensoredAtDeath = 1 and EpisodeEndDt > DeathDt then EpisodeEnddt = deathdt;

	        *Trucate episode at IOT;
			*Only adjust for IOT when IOT prior to EpisodeEndDate. IOT after EpisodeEndDt can 
			 occur if Epiosde shortened to occur on EndDate;
	        if iot and TrunkDate and trunkdate <= EpisodeEndDt then EpisodeEndDt=TrunkDate;

	    	*ExpExtPer may have made the episode to go beyond elig.;
	    	if EpisodeEndDt > enr_end then EpisodeEndDt = enr_end;

	        if blackoutper =. then blackoutper=0;

	        *Minimum Episode Duration;
	        if EpisodeEndDt-IndexDt + 1 >= minepisdur then MinEpiDurMet=1;  

			*Maximum Episode Duration;
	        if not missing(maxepisdur) and EpisodeEndDt-IndexDt + 1 > maxepisdur then EpisodeEndDt = IndexDt + maxepisdur -1;  

	        if MinEpiDurMet;

	    	*Defensive coding;
	    	if IndexDt > EpisodeEndDt then delete;                

	        keep PatID IndexDt EpisodeEndDt ENR_START ENR_END 
	             mindaysupp FupWashPer blackoutper T&type.CohortDef IndexLook washper RxSup RxAmt NumDispensing
		 			OrigEpisEndDt trunkdate EnrEpisodeCensoredAtDeath maxepisdur minepisdur Deathdt enr_end_equal_death
					code productapprovaldate productmarketingdate otherproductdate uptakedate INCEXL;
		run;

 		%ms_shaveoutside(reffile=_PtsMasterList2,
	                     refstart=IndexDt,
	                     refend=EpisodeEndDt,
	                     KeepPartBf=Y,
	                     ToShaveFile=_GroupIndex2,
	                     ToShaveStart=Adate,
	                     ToShaveEnd=ExpireDt,
	                     outfile=_GroupUtilFile2);

	    *Summarize RxSup during episode (only to apply mindaysupp criterion);
	    *The final summarization will be done later when episode selection is finalized;
	    proc means data=_GroupUtilFile2 nway noprint;
	    var RxSup RxAmt;
	    class PatId IndexDt;
	    output out=_GroupUtilFile2(keep=PatId IndexDt TotRxSup) sum(RxSup)=TotRxSup;
	    run;

	    data _PtsMasterList2;
			merge _PtsMasterList2(in=a)
		          _GroupUtilFile2;
		    by PatId IndexDt;
		    if a;

		    if TotRxSup >= mindaysupp then MinDaySupMet=1;
		    if MinDaySupMet;

		    drop TotRxSup washper;
		run;

		*Identify prevalent/incident episodes;
		data _PtsMasterList;
			merge _PtsMasterList(in=inc keep=patid indexdt)
				_PtsMasterList2 (in=prev);
			by patid indexdt;

			format PrevInc $5.;
			if prev and not inc then PrevInc = "Prev";
			else PrevInc = "Inc";
		run;
	%end; /*Type 6*/

	****************************************;
	* Type 3 cohort definition		 		;
	****************************************;
 	%IF %EVAL(&TYPE.=3) %THEN %DO;

		data _null_;
		call symputx("ExpPeriodEndDt",INTNX('Month', &T3ENDOFUPDATE.-max(&T3RiskTo.,&T3CtrlTo.,0, &enrdaysaftInd.),0, 'begin')-1);  *for inclusion in metadatafile; 		
		run;

		data _null_;
		call symputx("MaxExpPeriodEndDt",Max(&MaxExpPeriodEndDt., &ExpPeriodEndDt.));
		run;

        *Manage All Dates;
        data _POV1;
        set _POV1;
        by PatId;
		format T3CtrlFromDt T3CtrlToDt T3RiskFromDt T3RiskToDt MinWindowDt MaxWindowDt date9.;

        *Calulate windows start and end dates for reuse;        
        T3CtrlFromDt=Adate+T3CtrlFrom;
        T3CtrlToDt=Adate+T3CtrlTo;
        T3RiskFromDt=Adate+T3RiskFrom;
        T3RiskToDt=Adate+T3RiskTo;

        *Calculate length of lookback period for HOI INCIDENCE ASSESSMENT;            
        MinWindowDt=min(ADate, T3CtrlFromDt,T3RiskFromDt);
        MaxWindowDt=max(ADate, T3CtrlToDt,T3RiskToDt);
        LookBackLength=MaxWindowDt-MinWindowDt+1;
        
		RequiredFUPPer=max(T3RiskTo,T3CtrlTo,0, &enrdaysaftind.);
		call symputx("ExpPeriodEndDt",INTNX('Month', &T3ENDOFUPDATE.-RequiredFUPPer,0, 'begin')-1);  *for inclusion in metadatafile; 

        *Calculate min/max elig requirement from Adate perspective and using LookBackLength;
        *Note: Exclusion elig requirement is already accounted for in POV3;
        format MinEligtDt MaxEligtDt MinEligtHOIDt MaxEligtEnrAftDt date9.;


		MinEligtDt=Adate-max(0, &enrdays., Washper,&MinCovFrom., &MinMFUFROM.);       
		MinEligtHOIDt=min(MinEligtDt, MinWindowDt - LookBackLength);
        MaxEligtDt=max(0,MaxWindowDt,Adate);

		MaxEligtEnrAftDt=.;
		if &enrdaysaftind. > 0 then MaxEligtEnrAftDt=Max(MaxEligtDt, Adate + &enrdaysaftind.);
		if T3FupWashPer = . then LookBackLength=999999;
	
		LookBackLength=max(LookBackLength,T3FupWashPer);
        run;

        *IndexDt Reference Cohort Criteria (Exposure Event Criteria);
        data _PtsMasterList;
		format IndexDt ENR_START ENR_END date9.;
        merge _POV1(in=pov1 rename=Adate=IndexDt)
              _POV2(in=pov2 rename=Adate=IndexDt)
              _POV3(in=pov3 rename=Adate=IndexDt keep=patid Adate)
              _AgeGroups(keep=PatId IndexDt age birth_date sex race hispanic AgeGroup:);
        by PatId IndexDt;
		
        if pov1 and not pov2 and pov3;

        *Minimum look back enrollment duration;
        if  Enr_Start <= MinEligtDt then MinEnrolMet=1;
		if  Enr_Start <= MinEligtHOIDt then MinEnrolHOIMet=1;
        if  Enr_End >= MaxEligtEnrAftDt then MinEnrolMet2=1;

        if  MinEnrolMet and MinEnrolMet2;        

		keep PatID IndexDt ENR_START ENR_END EnrEpisodeCensoredAtDeath MinEnrolHOIMet MinEnrolMet2 MaxEligtEnrAftDt
             MinEligtDt MaxEligtDt MinWindowDt MaxWindowDt LookBackLength age birth_date sex race hispanic AgeGroup:  
             T3FupWashPer T3CohortDef T3CtrlFromDt T3CtrlToDt T3RiskFromDt T3RiskToDt MinEligtHOIDt zip: state hhs_reg cb_reg;
        run;

    %END;
	
	****************************************;
	* Type 4 cohort definition		 		;
	****************************************;
	%IF %EVAL(&TYPE.=4) %THEN %DO;	

        *IndexDt Reference Cohort Criteria;
		*Output data for enroll table;
        data _PtsMasterList;
			format IndexDt EpisodeStartDt EpisodeEndDt ENR_START ENR_END date9.;
	        merge _POV1(in=pov1 rename=Adate=IndexDt)
	              _POV2(in=pov2 rename=Adate=IndexDt)
	              _POV3(in=pov3 rename=Adate=IndexDt keep=patid Adate)
	              _POV4(in=pov4 rename=EpisodeStartDt=IndexDt);              
	        by Patid IndexDt; * Adate represents potential Index Date;        

			if first.PatId then DelNum=1;
			else DelNum=DelNum+1;
			retain DelNum;

			if pov1 and not pov2 and pov3 and pov4;

			* Apply the period dates criterion to episode end date (unlike other types it is not applied to IndexDt);
			if &startfollowup. <= EpisodeEndDt <= &enddate.;
	          	
	        *Minimum look back enrollment duration;
	        if  Enr_Start <= EpisodeEndDt -max(0,&enrdays.,Washper,T4FupWashPer) then MinEnrolMet=1; 
			if  Enr_End >= EpisodeEndDt+max(0,&EnrDaysAftind.) then MinEnrolMet2=1;  

	    	*Defensive coding;
	    	if IndexDt > EpisodeEndDt then delete;
	           
			label PregDurCode="PregDurCode"; 
			
			*Create eligdays for enrolment tables;
			eligdays=adate-enr_start;

			drop Duration episodegap episodegap2 expextper expextper2 gap ittdays ittdays2 
				 LEnrStart LRunOutDate RXAMT RXSUP WashPer2  t4CohortDef2 t4cohortdef t4fupwashper2;		
		run;
	
		*Need to also ensure linked infant meets post birth enrollment requirement - first enrollment end date post birth;
		proc sql noprint;
			create table _linkedenr as
			select pts.linkedid
				 , min(enr.enr_end) as cenr_end
			from _PtsMasterList as pts
			left join dplocal.enr_&RECONTYP.&ENROLGAP. as enr
			on pts.linkedid = enr.patid
			where enr.enr_end >= pts.cenr_start
			group by linkedid;
		quit;

		proc sql noprint undo_policy=none;
			create table _PtsMasterList_linkenr as
			select pts.*,
				   enr.cenr_end
			from _PtsMasterList as pts
			left join _linkedenr as enr
			on pts.linkedid = enr.linkedid;
		quit;

		data _PtsMasterList
 			 _Deliveries(drop=adate birth_date has: priority: race hispanic sex indexdt expiredt episode
							  washper t4fupwashper ConcWashPer MinEnrolMet);

			set _PtsMasterList_linkenr;
			if missing(linkedID)=0 then do;
				if CEnr_End >= EpisodeEndDt+max(0,&EnrDaysAftind.) then MinEnrolMet3=1;  
			end;
			else do;
				MinEnrolMet3=1;  
			end;

			if  MinEnrolMet and MinEnrolMet2 and MinEnrolMet3 then output _PtsMasterList;
			output _Deliveries;
        run;

		proc sql noprint undo_policy=none;
        create table _PtsMasterList as
        select mstr.*,
               age.AgeGroup 
        from _PtsMasterList as mstr,
             _AgeGroups as age
        where mstr.PatId=age.PatId and mstr.EpisodeEndDt=age.IndexDt
        order by Patid,IndexDt;
        quit;

		proc sql noprint undo_policy=none;
        create table _Deliveries as
        select del.*,
               age.AgeGroup 
        from _Deliveries as del,
             _AgeGroups as age
        where del.PatId=age.PatId and del.EpisodeEndDt=age.IndexDt
        order by Patid,IndexDt;
        quit;

		data _POV1_Sec;
		format MOIName $10.;
		set 
		%do n=1 %to &NUM_MPN.;			
			_POV1&n.(in=file&n.)			
		%end;;
		%do n=1 %to &NUM_MPN.;	
			if file&n. then MOIName="MP&n.";	
		%end;
		run;

		data _POV2_Sec;
		format MOIName $10.;
		set 
		%do n=1 %to &NUM_MPN.;			
			_POV2&n.(in=file&n.)			
		%end;;
		%do n=1 %to &NUM_MPN.;	
			if file&n. then MOIName="MP&n.";	
		%end;
		run;	

		data _POV4_Sec;
		format MOIName $10.;
		set 
		%do n=1 %to &NUM_MPN.;			
			_POV4&n.(in=file&n.)			
		%end;;
		%do n=1 %to &NUM_MPN.;	
			if file&n. then MOIName="MP&n.";	
		%end;
		run;
		
		proc sql noprint;
		create table _PtsMasterList2 as
		select pov1.*,
			   pov4.Episode,
			   pov4.EpisodeStartDt as IndexDt,
			   pov4.EpisodeEndDt
		from _POV1_Sec as pov1,
			 _POV4_Sec as pov4 
		where pov1.PatId=pov4.PatId and
			  pov1.MOIName=pov4.MOIName and 
			  pov1.ADate=pov4.EpisodeStartDt			  
		order by PatId, MOIName, ADate;
		quit;

		proc sort data=_POV2_Sec;
		by PatId MOIName ADate;
		run;

		data _PtsMasterList2;
		merge _PtsMasterList2 (in=a)
		      _POV2_Sec(in=pov2);
		by PatId MOIName ADate;  * Adate represents potential Index Date;
		if a and not pov2;
		drop ConcWashPer DaysUntreated episodegap: expextper: ExpireDt ittdays ittdays2 t4cohortdef t4CohortDef2
		t4fupwashper t4fupwashper2 WashPer WashPer2 ADate RxSup RxAmt CodeSupply;
		run;
	
    %END;

	proc datasets nowarn noprint lib=work;
		delete ENR_&RECONTYP.&ENROLGAP.;
	quit;

%put NOTE: ********END OF MACRO: ms_createptsmasterlist v1.10********;

%mend ms_createptsmasterlist;
